@@ -2,7 +2,7 @@
 #
 # Shape - A class for to represent Excel XLSX shape objects.
 #
-# Copyright 2013-2015, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
 #
 import copy
 from warnings import warn
@@ -34,6 +34,7 @@ class Shape(object):
         self.edit_as = ''
         self.id = 0
         self.text = ''
+        self.textlink = ''
         self.stencil = 1
         self.element = -1
         self.start = None
@@ -46,6 +47,7 @@ class Shape(object):
         self.flip_h = 0
         self.flip_v = 0
         self.rotation = 0
+        self.text_rotation = 0
         self.textbox = False
 
         self.align = None
@@ -53,6 +55,8 @@ class Shape(object):
         self.font = None
         self.format = None
         self.line = None
+        self.url_rel_index = None
+        self.tip = None
 
         self._set_options(options)
 
@@ -70,6 +74,12 @@ class Shape(object):
         self.gradient = self._get_gradient_properties(options.get('gradient'))
         self.line = self._get_line_properties(options.get('line'))
 
+        self.text_rotation = options.get('text_rotation', 0)
+
+        self.textlink = options.get('textlink', '')
+        if self.textlink.startswith('='):
+            self.textlink = self.textlink.lstrip('=')
+
         if options.get('border'):
             self.line = self._get_line_properties(options['border'])
 
@@ -311,7 +321,6 @@ class Shape(object):
             'pitch_family': options.get('pitch_family'),
             'charset': options.get('charset'),
             'baseline': options.get('baseline', -1),
-            'rotation': options.get('rotation'),
             'lang': options.get('lang', 'en-US'),
         }
 
@@ -319,10 +328,6 @@ class Shape(object):
         if font['size']:
             font['size'] = int(font['size'] * 100)
 
-        # Convert rotation into 60,000ths of a degree.
-        if font['rotation']:
-            font['rotation'] = 60000 * int(font['rotation'])
-
         return font
 
     @staticmethod
