@@ -6,6 +6,7 @@
 #
 
 # Standard packages.
+import hashlib
 import re
 import os
 import operator
@@ -631,7 +632,7 @@ class Workbook(xmlwriter.XMLwriter):
             raise e
 
         # Assemble worksheets into a workbook.
-        packager = Packager()
+        packager = self._get_packager()
 
         # Add a default worksheet if non have been added.
         if not self.worksheets():
@@ -1101,7 +1102,10 @@ class Workbook(xmlwriter.XMLwriter):
         # Iterate through the worksheets and set up chart and image drawings.
         chart_ref_id = 0
         image_ref_id = 0
+        ref_id = 0
         drawing_id = 0
+        image_ids = {}
+        header_image_ids = {}
 
         for sheet in self.worksheets():
             chart_count = len(sheet.charts)
@@ -1125,12 +1129,19 @@ class Workbook(xmlwriter.XMLwriter):
             for index in range(image_count):
                 filename = sheet.images[index][2]
                 image_data = sheet.images[index][10]
-                (image_type, width, height, name, x_dpi, y_dpi) = \
+                (image_type, width, height, name, x_dpi, y_dpi, md5) = \
                     self._get_image_properties(filename, image_data)
-                image_ref_id += 1
 
-                sheet._prepare_image(index, image_ref_id, drawing_id, width,
-                                     height, name, image_type, x_dpi, y_dpi)
+                if md5 in image_ids:
+                    ref_id = image_ids[md5]
+                else:
+                    image_ref_id += 1
+                    ref_id = image_ref_id
+                    image_ids[md5] = image_ref_id
+                    self.images.append([filename, image_type, image_data])
+
+                sheet._prepare_image(index, ref_id, drawing_id, width, height,
+                                     name, image_type, x_dpi, y_dpi, md5)
 
             # Prepare the worksheet charts.
             for index in range(chart_count):
@@ -1148,14 +1159,20 @@ class Workbook(xmlwriter.XMLwriter):
                 image_data = sheet.header_images[index][1]
                 position = sheet.header_images[index][2]
 
-                (image_type, width, height, name, x_dpi, y_dpi) = \
+                (image_type, width, height, name, x_dpi, y_dpi, md5) = \
                     self._get_image_properties(filename, image_data)
 
-                image_ref_id += 1
+                if md5 in header_image_ids:
+                    ref_id = header_image_ids[md5]
+                else:
+                    image_ref_id += 1
+                    ref_id = image_ref_id
+                    header_image_ids[md5] = image_ref_id
+                    self.images.append([filename, image_type, image_data])
 
-                sheet._prepare_header_image(image_ref_id, width, height,
+                sheet._prepare_header_image(ref_id, width, height,
                                             name, image_type, position,
-                                            x_dpi, y_dpi)
+                                            x_dpi, y_dpi, md5)
 
             # Prepare the footer images.
             for index in range(footer_image_count):
@@ -1164,14 +1181,20 @@ class Workbook(xmlwriter.XMLwriter):
                 image_data = sheet.footer_images[index][1]
                 position = sheet.footer_images[index][2]
 
-                (image_type, width, height, name, x_dpi, y_dpi) = \
+                (image_type, width, height, name, x_dpi, y_dpi, md5) = \
                     self._get_image_properties(filename, image_data)
 
-                image_ref_id += 1
+                if md5 in header_image_ids:
+                    ref_id = header_image_ids[md5]
+                else:
+                    image_ref_id += 1
+                    ref_id = image_ref_id
+                    header_image_ids[md5] = image_ref_id
+                    self.images.append([filename, image_type, image_data])
 
                 sheet._prepare_header_image(image_ref_id, width, height,
                                             name, image_type, position,
-                                            x_dpi, y_dpi)
+                                            x_dpi, y_dpi, md5)
 
             if has_drawing:
                 drawing = sheet.drawing
@@ -1203,6 +1226,8 @@ class Workbook(xmlwriter.XMLwriter):
             # Read the image data from the user supplied byte stream.
             data = image_data.getvalue()
 
+        md5 = hashlib.md5(data).hexdigest()
+
         # Get the image filename without the path.
         image_name = os.path.basename(filename)
 
@@ -1246,9 +1271,6 @@ class Workbook(xmlwriter.XMLwriter):
             raise UndefinedImageSize(
                 "%s: no size data found in image file." % filename)
 
-        # Store image data to copy it into file container.
-        self.images.append([filename, image_type, image_data])
-
         if not image_data:
             fh.close()
 
@@ -1258,7 +1280,7 @@ class Workbook(xmlwriter.XMLwriter):
         if y_dpi == 0:
             y_dpi = 96
 
-        return image_type, width, height, image_name, x_dpi, y_dpi
+        return image_type, width, height, image_name, x_dpi, y_dpi, md5
 
     def _process_png(self, data):
         # Extract width and height information from a PNG file.
@@ -1632,6 +1654,11 @@ class Workbook(xmlwriter.XMLwriter):
         # Convert the SST string data from a dict to a list.
         self.str_table._sort_string_data()
 
+    def _get_packager(self):
+        # Get and instance of the Packager class to create the xlsx package.
+        # This allows the default packager to be over-ridden.
+        return Packager()
+
     ###########################################################################
     #
     # XML methods.
